#include "devboard.h"

void SetPAR( int mode )
{
	short int* psiPAR;
	
	psiPAR = (short int*) (MBASE+1+PAR);
	
	if(mode == 0)
		/* Set Parallel Port (PPX) to Debug Mode */
		*psiPAR = 0x30;
	else if (mode == 1)
		/* Set Parallel Port (PPX) to General Purpose I/O Mode */
		*psiPAR = 0x00;
	else
		/* Set Parallel Port (PPX) to User Define Mode */
		*psiPAR = mode;
}

void Delay(int factor)
{
	int delay;
	
	for(delay=0; delay < (factor*7176);delay++);
}

void RunningLight()
{
	int light;
	char* pDir;
	char* pData;
	
	SetPAR(1);
	
	/* Set Parallel Port as output */
	pDir = (char*) (MBASE+PPDDR);
	*pDir = 0xFF;
	
	/* Dump initial data to Parallel Port */
	pData = (char*) (MBASE+PPDAT);
	*pData = 0x0;

	/* Running Light on PP0 -> PP7 */
	light = 1;
	
	while(1)
	{
		if(light == 0x100)
			light = 1;
		
		*pData = (char)light;
		light <<= 1;
		
		Delay(10);
	}
}

void Counter(int size, int write, int dump)
{
	char* pAddr;
	char* pData;
	char* pDir;

	int count;
	int max;
	int maxaddr;
	
	max = (size>(ADREND+1-ADRBASE)?(ADREND+1-ADRBASE):(size+1));
	
	if(write!=0)
	{
		pAddr = (char*) ADRBASE;
	
		/* Write data to ADRAM */
		for(count=0x00000;count<max;count++)
		{
			*pAddr = (char)(count%0x100);	
			pAddr++;
		}
	}
	
	if(dump)
	{
		SetPAR(1);
		
		/* Set Parallel Port as output */
		pDir = (char*) (MBASE+PPDDR);
		*pDir = 0xFF;
		
		/* Load data from ADRAM */
		for(count=0x0;count<max;count++)
		{
			pAddr = (char*) (ADRBASE+count);
			/* Dump data to Parallel Port */
			pData = (char*) (MBASE+PPDAT);
			*pData = *pAddr;
			Delay(2);
		}
	}
}
	
void WritePP(int data)
{
	char* pDir;
	char* pData;
	
	SetPAR(1);
	
	/* Set Parallel Port as output */
	pDir = (char*) (MBASE+PPDDR);
	*pDir = 0xFF;

	/* Dump data to Parallel Port */		
	pData = (char*) (MBASE+PPDAT);
	*pData = (char) data;
}

void WriteADRAM(int addr, int data)
{
	char* pAddr;
	
	pAddr = (char*) (addr);
	
	/* Write data to ADRAM */
	*pAddr = (char) data;	
}

int ReadADRAM(int addr, int write)
{
	char* pAddr;
	
	pAddr = (char*) (addr);
	
	if(write)
		WritePP(*pAddr);
	
	return (int)*pAddr;
}

int SetADRAM(int start, int end, int data)
{
	int max;
	int count;
	char* pAddr;
	
	pAddr = (char*) ADRBASE;
	
	/* Write data to ADRAM */
	for(count=start;count<(end+1);count++)
	{
		*pAddr = (char)data;	
		pAddr++;
	}
}

void SetPPReg(int data)
{
	char* pData;
	
	pData = (char*)(MBASE+PPDAT);
	*pData = (char) data;
}

int GetPPReg()
{
	char* pData;
	
	pData = (char*)(MBASE+PPDAT);
	return (int)(*pData);
}
