#include <stdio.h>
#include "mblaster.h"
#include "io.h"

int MBlaster(int argc, char* argv[])
{
	int file_id;

	/* Syntax check */
	if ( argc != 2 )
	{
		Result(1);
		while(1);
	}

	/* Open programming file as READ and in BINARY */
	file_id = fopen_rbf( argv[1], "rb" );

	if ( !file_id )
	{
		Result(1);
		while(1);
	}


	/* Processing and Programming */
	/*file_id =*/ ProcessFileInput( file_id );

	if ( file_id )
		fclose_rbf(file_id);

	return file_id;
}

void ProcessFileInput( int finputid )
{
	int			program_done = 0;			/* programming process (configuration and initialization) done = 1 */
	int			seek_position = 0;			/* file pointer position */
	long int	file_size = 0;				/* programming file size */
	int			configuration_count = RECONF_COUNT_MAX;	/* # reprogramming after error */
	int			one_byte = 0;				/* the byte to program */
	long int	i = 0;						/* standard counter variable */
	int			confdone_ok = 1;			/* CONF_DONE pin. '1' - error */
	int			nstatus_ok = 0;				/* NSTATUS pin. '0' - error */
	int			clock_x_cycle = CLOCK_X_CYCLE; /* Clock another 'x' cycles during initialization (optional) */

	/* Get file size */
	seek_position = fseek_rbf( finputid, 0, S_END );

	if ( seek_position )
	{
		Result(1);
		while(1);
	}
	
	file_size	= ftell_rbf( finputid );

	/* Start configuration */
	while ( !program_done && (configuration_count>0) )
	{
		/* Reset file pointer and parallel port registers */
		fseek_rbf( finputid, 0, S_SET );
		SetPortMode( PM_RESET );

		/* Drive a transition of 0 to 1 to NCONFIG to indicate start of configuration */
		Dump2Port( SIG_NCONFIG, 0, 0 );
		Dump2Port( SIG_NCONFIG, 1, 0 );


		Burst( file_size);
		program_done = 1;
		
		/* Loop through every single byte */

		configuration_count--;

		if ( !program_done )
			continue;

		/* Configuration end */
		/* Check CONF_DONE that indicates end of configuration */
	
		confdone_ok = CheckSignal( SIG_CONFDONE );

		if ( confdone_ok )
		{
			Result(1);
			program_done = 0;
			PrintError( configuration_count );			
			if ( configuration_count == 0 )
				break;
		}
	
		/* if contain error during configuration, restart configuration */
		if ( !program_done )
			continue;

		/* Start initialization */
		/* Clock another (INIT_CYCLE) cycles to initialize the device */
		for ( i = 0; i < INIT_CYCLE; i++ )
		{
			Dump2Port( SIG_DCLK, 0, 0 );
			Dump2Port( SIG_DCLK, 1, 0 );
		}
		/* Initialization end */

		nstatus_ok = CheckSignal( SIG_NSTATUS );
		confdone_ok = CheckSignal( SIG_CONFDONE );

		if ( !nstatus_ok || confdone_ok )
		{
			Result(1);
			program_done = 0; 
			configuration_count = 0; /* No reconfiguration */
		}
	}

	/* add another 'x' clock cycles to make sure the device is initialized (certain cases) */
	if ( clock_x_cycle > 0 )
	{		
		for ( i = 0; i < CLOCK_X_CYCLE; i++ )
		{
			Dump2Port( SIG_DCLK, 0, 0 );
			Dump2Port( SIG_DCLK, 1, 0 );
		}
	}

	if ( !program_done )
	{
		Result(1);
		SetPortMode( PM_RESET );
		return;
	}
	
	SetPortMode( PM_USER );
}

void SetPortMode( int mode )
{
	/* write to Port 0 and Port 2 with predefined values */
	port_data[0] = port_mode_data[mode][0];
	port_data[2] = port_mode_data[mode][2];
	WriteByteBlaster( 0, port_data[0], 1 );
	WriteByteBlaster( 2, port_data[2], 1 );
}

void Dump2Port( int signal, int data, int clk )
{
	/* Get signal port number */
	int port = sig_port_maskbit[signal][0];

	/* Get signal mask bit*/
	int mask;
	
	/* If clk == 1, drive signal with [data] and drive SIG_DCLK with '0' together. Then drive SIG_DCLK with '1' */
	/* That is to create a positive edge pulse */
	if ( clk )
		mask = sig_port_maskbit[signal][1] | sig_port_maskbit[SIG_DCLK][1];
	else
		mask = sig_port_maskbit[signal][1];
	
	/* AND signal bit with '0', then OR with [data] */
	mask = ~mask;
	port_data[port] = ( port_data[port] & mask ) | ( data * sig_port_maskbit[signal][1] );
	
	WriteByteBlaster( port, port_data[port], 0 );

	if ( clk ) WriteByteBlaster( port, ( port_data[port] | sig_port_maskbit[SIG_DCLK][1] ), 0 );
}

void ProgramByte( int one_byte )
{
	int	bit = 0;
	int i = 0;

	one_byte = fgetc_rbf(one_byte);
	
	/* write from LSb to MSb */
	for ( i = 0; i < 8; i++ )
	{
		bit = (one_byte >> i) & 0x1;
			
		/* Dump to DATA0 and insert a positive edge pulse at the same time */
		Dump2Port( SIG_DATA0, bit, 1 );
	}
}

int CheckSignal( int signal )
{
	int value = 1;
	int	readout = 0;

	readout = ReadByteBlaster( 1 );
	value = readout & sig_port_maskbit[signal][1];

	return value;
}

void PrintError( int configuration_count )
{
	if ( configuration_count == 0 )
		fprintf( stderr, "Error: Error in configuration #%d... \nError: Maximum number of reconfiguration reached. Exiting...\n", (RECONF_COUNT_MAX-configuration_count) );
	else
	{
		fprintf( stderr, "Error: Error in configuration #%d... Restart configuration. Ready? <Press any key to continue>\n", (RECONF_COUNT_MAX-configuration_count) );
#if PORT == WINDOWS_NT
		getch();
#endif
	}
}
